package com.sample;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.h2.tools.Server;
import org.jbpm.bpmn2.xml.XmlBPMNProcessDumper;
import org.jbpm.process.core.event.EventFilter;
import org.jbpm.process.core.event.EventTypeFilter;
import org.jbpm.ruleflow.core.RuleFlowProcess;
import org.jbpm.ruleflow.core.RuleFlowProcessFactory;
import org.jbpm.runtime.manager.impl.RuntimeEnvironmentBuilder;
import org.jbpm.services.task.identity.JBossUserGroupCallbackImpl;
import org.jbpm.test.JBPMHelper;
import org.jbpm.workflow.core.node.EventNode;
import org.jbpm.workflow.core.node.HumanTaskNode;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.KieServices;
import org.kie.api.builder.KieFileSystem;
import org.kie.api.builder.ReleaseId;
import org.kie.api.io.Resource;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.manager.RuntimeEngine;
import org.kie.api.runtime.manager.RuntimeEnvironment;
import org.kie.api.runtime.manager.RuntimeManager;
import org.kie.api.runtime.manager.RuntimeManagerFactory;
import org.kie.api.runtime.process.ProcessInstance;
import org.kie.api.task.TaskService;
import org.kie.api.task.UserGroupCallback;
import org.kie.api.task.model.TaskSummary;
import org.kie.internal.runtime.manager.context.EmptyContext;
import org.kie.internal.runtime.manager.context.ProcessInstanceIdContext;

import bitronix.tm.resource.jdbc.PoolingDataSource;

/**
 * This is a sample file to launch a process.
 */
public class FluentBoundaryTimerTest {

    private static EntityManagerFactory emf;

    private static Server h2Server;
    private static PoolingDataSource ds;

    @Before
    public void setup() {

        // for H2 datasource
        h2Server = JBPMHelper.startH2Server();
        ds = JBPMHelper.setupDataSource();

        Map configOverrides = new HashMap();
        configOverrides.put("hibernate.hbm2ddl.auto", "create");
        configOverrides.put("hibernate.dialect", "org.hibernate.dialect.H2Dialect");

        emf = Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa", configOverrides);
    }

    @After
    public void teardown() {
        if (ds != null) {
            ds.close();
        }
        if (h2Server != null) {
            h2Server.shutdown();
        }
    }

    @Test
    public void testProcess() throws Exception {

        try {
            
            RuleFlowProcessFactory factory = RuleFlowProcessFactory.createProcess("com.sample.process");
            factory
                // header
                .name("MyProcess")
                .packageName("com.sample")
                
                // nodes
                .startNode(1)
                .name("Start")
                .done()
    
                .humanTaskNode(2)
                .name("Task")
                .actorId("john")
                .taskName("MyTask")
                .done()
    
                .endNode(3)
                .name("End")
                .terminate(true)
                .done()
    
                .eventNode(4)
                .name("TimerEvent")
                .eventType("TimerXXX") // Place holder to be swapped later because "attachedTo" is unknown at this moment
                .done()
    
                .actionNode(5)
                .name("Action")
                .action("java", "System.out.println(\"Time Up....\");")
                .done()
    
                .endNode(6)
                .name("End")
                .terminate(false)
                .done()
    
                .connection(1, 2)
                .connection(2, 3)

                .connection(4, 5)
                .connection(5, 6);
            
            RuleFlowProcess process = factory.validate().getProcess();
            
            // set meta data parameters and attach the filter for BoundaryTimer
            HumanTaskNode humanTaskNode = (HumanTaskNode)process.getNode(2);
            EventNode eventNode = (EventNode)process.getNode(4);
            
            String nodeAttachedTo = (String)humanTaskNode.getMetaData("UniqueId");
            
            String timeCycle = "3s###3s";
            
            eventNode.setMetaData("AttachedTo", nodeAttachedTo);
            eventNode.setMetaData("CancelActivity", false);
            eventNode.setMetaData("TimeCycle", timeCycle);
            EventTypeFilter eventFilter = new EventTypeFilter();
            eventFilter.setType("Timer-" + nodeAttachedTo + "-" + timeCycle);
            List<EventFilter> eventFilters = new ArrayList<EventFilter>();
            eventFilters.add(eventFilter);
            eventNode.setEventFilters(eventFilters);
            process.addNode(eventNode);

            KieServices ks = KieServices.Factory.get();
            KieFileSystem kfs = ks.newKieFileSystem();
            Resource resource = ks.getResources().newByteArrayResource(
                    XmlBPMNProcessDumper.INSTANCE.dump(process).getBytes());
            resource.setSourcePath("fluentProcessExample.bpmn2");
            kfs.write(resource);
            ReleaseId releaseId = ks.newReleaseId("com.sample", "fluentProcessExample", "1.0");
            kfs.generateAndWritePomXML(releaseId);
            ks.newKieBuilder(kfs).buildAll();
            KieBase kbase = ks.newKieContainer(releaseId).getKieBase();
            
            RuntimeManager manager = getRuntimeManager(kbase);
            
            long piid = -1;
            
            // 1st access
            {
                RuntimeEngine runtime = manager.getRuntimeEngine(EmptyContext.get());
                KieSession ksession = runtime.getKieSession();
    
                // start a new process instance
                Map<String, Object> params = new HashMap<String, Object>();
                params.put("employee", "john");
                params.put("reason", "Good job");
                ProcessInstance pi = ksession.startProcess("com.sample.process", params);
                piid = pi.getId();
                System.out.println("A process instance started : pid = " + piid);

                manager.disposeRuntimeEngine(runtime);
            }

            Thread.sleep(6000);
            
            // 2nd access. Now do human task
            {
                RuntimeEngine runtime = manager.getRuntimeEngine(ProcessInstanceIdContext.get(piid));
                TaskService taskService = runtime.getTaskService();

                List<TaskSummary> list = taskService.getTasksAssignedAsPotentialOwner("john", "en-UK");
                for (TaskSummary taskSummary : list) {
                    System.out.println("john starts a task : taskId = " + taskSummary.getId());
                    taskService.start(taskSummary.getId(), "john");
                    taskService.complete(taskSummary.getId(), "john", null);
                }

                manager.disposeRuntimeEngine(runtime);
            }
            
        } catch (Throwable th) {
            th.printStackTrace();
        }
    }

    private static RuntimeManager getRuntimeManager(KieBase kbase) {
        Properties properties = new Properties();
        properties.setProperty("krisv", "");
        properties.setProperty("mary", "");
        properties.setProperty("john", "");
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);

        RuntimeEnvironment environment = RuntimeEnvironmentBuilder.getDefault().persistence(true)
                .entityManagerFactory(emf).userGroupCallback(userGroupCallback)
                .knowledgeBase(kbase)
                .get();
        return RuntimeManagerFactory.Factory.get().newPerProcessInstanceRuntimeManager(environment);

    }
}